package com.mcwl.memberCenter.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.memberCenter.domain.MemberBenefit;
import com.mcwl.memberCenter.domain.MemberLevel;
import com.mcwl.memberCenter.domain.Member;
import com.mcwl.memberCenter.enums.MemberBenefitTypeEnum;
import com.mcwl.memberCenter.enums.MemberEnum;
import com.mcwl.memberCenter.enums.MemberPeriodicEnum;
import com.mcwl.memberCenter.mapper.MemberMapper;
import com.mcwl.memberCenter.service.MemberBenefitService;
import com.mcwl.memberCenter.service.MemberLevelService;
import com.mcwl.memberCenter.service.MemberService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.*;

@Service
@RequiredArgsConstructor
public class MemberServiceImpl extends ServiceImpl<MemberMapper, Member> implements MemberService {

    private final MemberLevelService memberLevelService;

    private final ISysUserService sysUserService;

    private final MemberBenefitService memberBenefitService;

    @Override
    public Member createUserMember(Long userId, Long memberLevelId, String paymentMethod) {
        if (userId == null) {
            return null;
        }
        SysUser user = sysUserService.selectUserById(userId);
        if (user == null || memberLevelId == null) {
            return null;
        }
        MemberLevel memberLevel = memberLevelService.getById(memberLevelId);
        if (memberLevel == null) {
            return null;
        }
        // 查询用户是否已经存在会员
        LambdaQueryWrapper<Member> lqw = new LambdaQueryWrapper<>();
        lqw.eq(Member::getUserId, userId)
                // subscriptionStatus不为expired
                .ne(Member::getSubscriptionStatus, MemberEnum.MEMBER_CENTER_EXPIRED)
                .orderBy(true, false, Member::getEndDate);
        List<Member> memberList = baseMapper.selectList(lqw);
        Member member = new Member();
        // 设置用户id
        member.setUserId(userId);
        // 设置会员id
        member.setMemberLevelId(memberLevelId);
        if (memberList != null && !memberList.isEmpty()) {
            Member lastMember = memberList.get(0);
            Date endDate = lastMember.getEndDate();
            // 设置会员开始时间和结束时间
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(endDate);
            calendar.add(Calendar.DAY_OF_MONTH, 1);
            member.setStartDate(calendar.getTime());
            member.setEndDate(this.getEndDate(memberLevel.getSubscriptionPeriod(), calendar));
        } else {
            // 用户不存在会员
            // 设置会员开始时间和结束时间
            Calendar calendar = Calendar.getInstance();
            member.setStartDate(calendar.getTime());
            member.setEndDate(this.getEndDate(memberLevel.getSubscriptionPeriod(), calendar));
        }

        // 设置积分
        MemberBenefit memberBenefit = memberBenefitService.lambdaQuery()
                .eq(MemberBenefit::getMemberLevelId, memberLevelId)
                .eq(MemberBenefit::getBenefitType, MemberBenefitTypeEnum.POINTS)
                .one();
        if (memberBenefit != null) {
            member.setPoints(memberBenefit.getBenefitDiscount());
        }

        // 设置订阅状态
        if (memberLevel.getSubscriptionPeriod() == MemberPeriodicEnum.CONTINUE_MONTH) {
            // 连续包月，会员状态为活跃
            member.setSubscriptionStatus(MemberEnum.MEMBER_CENTER_ACTIVE);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(member.getEndDate());
            calendar.add(Calendar.MONTH, 1);
            member.setNextBillingDate(calendar.getTime());
        } else {
            member.setSubscriptionStatus(MemberEnum.MEMBER_CENTER_INACTIVE);
        }
        // 设置支付时间
        member.setLastPaymentDate(new Date());
        // 设置支付方式
        member.setPaymentMethod(paymentMethod);
        // 设置创建者
        SysUser sysUser = sysUserService.selectUserById(userId);
        member.setCreateBy(sysUser.getUserName());
        member.setCreateTime(new Date());
        member.setUpdateBy(sysUser.getUserName());
        member.setUpdateTime(new Date());

        baseMapper.insert(member);


        return member;
    }


    @Override
    public List<Member> getUseUserMember() {
        return this.getUseUserMember(null);
    }

    @Override
    public Member getUseUserMemberByUserId(Long userId) {

        List<Member> memberList = this.getUseUserMember(userId);
        if (memberList == null || memberList.isEmpty()) {
            return null;
        }
        return memberList.get(0);
    }

    @Override
    public Member rechargePoints(Long userId, Double points) {
        if (userId == null) {
            return null;
        }
        if (points == null) {
            return null;
        }
        Member member = getUseUserMemberByUserId(userId);
        if (member != null) {
            member.setPoints(member.getPoints() + points);
            baseMapper.updateById(member);
            return member;
        }
        return null;
    }

    private List<Member> getUseUserMember(Long userId) {
        // startDate 小于等于当前时间、endDate 大于等于当前时间
        // subscriptionStatus 不为 "过期" 或 "待支付"
        // status 为 0 的
        LambdaQueryWrapper<Member> qw = new LambdaQueryWrapper<>();
        qw.le(Member::getStartDate, new Date())
                .ge(Member::getEndDate, new Date())
                .ne(Member::getSubscriptionStatus, MemberEnum.MEMBER_CENTER_EXPIRED)
                .ne(Member::getSubscriptionStatus, MemberEnum.MEMBER_CENTER_PENDING)
                .eq(Member::getStatus, "0")
                .eq(userId != null, Member::getUserId, userId);
        return baseMapper.selectList(qw);
    }


    /**
     * 根据订阅周期和开始时间 计算结束时间
     * @param subscriptionPeriod 订阅周期
     * @param calendar 结束时间（日历）
     * @return 结束时间
     */
    private Date getEndDate(MemberPeriodicEnum subscriptionPeriod, Calendar calendar) {
        switch (subscriptionPeriod) {
            case YEAR:
                calendar.add(Calendar.YEAR, 1);
                break;
            case QUARTER:
                calendar.add(Calendar.MONTH, 3);
                break;
            case MONTH:
            case CONTINUE_MONTH:
                calendar.add(Calendar.MONTH, 1);
                break;
            default:
                break;
        }
        return calendar.getTime();
    }

}
